# YOUnite File System Adaptor

## Overview

The File System Adaptor is actually two types of adaptors: a source and a destination. The source adaptor monitors
folders for changes and sends them to YOUnite. The destination adaptor receives messages from YOUnite and performs
the file operation.

The source and destination cannot be the same adaptor.

For a quick start on how to run the File System / S3 demo in conjunction with the YOUnite stack and the `Biz demo` see link:FS-S3-Local-Demo.html[File System and S3 Demo].

## Features

* File system support: everything Apache VFS supports as well as Amazon S3.
* Fast backing database (H2) runs on the local file system, ensures that on restart or recovery from failure that
file system monitoring resumes where it left off.
* Scanning and processing changes are asynchronous, for efficiency. 

## Limitations

* Entire list of monitored files is stored in memory during monitoring. It is backed by a database only for restart
or recovery from failure.
* H2 is only currently supported database.

### File Name Limitations

* File names can contain nearly any character **EXCEPT** forward slash "/" and percent sign "%". Otherwise, as long
as the file system can handle it, any other character should be acceptable.
 
## Testing notes

For integration tests to work, AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY must be set, or a ~/.aws/profile present
with valid credentials.

## Configuration

All configuration items may be specified as environment variables (usually uppercase with underscores delimiting words)
or put in a file `/adaptor.properties` (usually lowercase with periods delimiting words).

### Adaptor Base Configuration

The following configuration options are common to all types of adpators built on the adaptor SDK.

Required properties:

|Property |Environment Variable |Description |Example Value
|---------|---------------------|------------|-------------|
|adaptor.uuid |ADAPTOR_UUID | UUID of the adaptor |`3dfcc03d-e5d4-4d57-9e9b-5c5d2db32f9a`
|client.id |CLIENT_ID |Client ID for the Adaptor - used to connect to message bus |`8c9167a6-bb83-4f77-bdfc-1947a946f77b`
|client.secret |CLIENT_SECRET |Client Secret for the Adaptor - used to connect to message bus |`de02e3fa-4b23-46cb-aed6-5665a16e73d3`
|message.bus.url |MESSAGE_BUS_URL |Message Bus URL |`nio+ssl://message-bus-uri:61617`

Optional properties:

|Property |Environment Variable |Description |Default Value
|---------|---------------------|------------|-------------|
|concurrency |CONCURRENCY |Message Bus concurrency for receiving and processing incoming events. | 1
|session.cache.size |SESSION_CACHE_SIZE |JMS session cache size. | 10
|enable.metrics |ENABLE_METRICS |Enable / disable displaying metrics at a given interval. | true
|metrics.log.interval |METRIC_LOGS_INTERVAL |Interval in milliseconds to display metrics. | 3600000 (one hour)
|n/a |LOG_LEVEL |Log level for YOUnite logback messages. Applies to `YOUnite DB adaptor and File System adaptor` implementations, but not the Adaptor SDK. Must come from an environment variable. | INFO
|n/a |ROOT_LOG_LEVEL |Log level for non-YOUnite logback messages. Applies to `YOUnite DB adaptor and File System adaptor` implementations, but not the Adaptor SDK. Must come from an environment variable. | INFO

### File System Adaptor Specific Configuration Options

Optional properties:

|Property |Environment Variable |Description |Default Value
|---------|---------------------|------------|-------------|
|data.directory |DATA_DIRECTORY | Directory where databases are stored | /data in the Docker image, or home (~) when run locally
|dr.id.database.name |DR_ID_DATABASE_NAME | Name of the file, without extension, of the H2 database in DATA_DIRECTORY for the DR to ID cache | dr_id_cache
|dr.id.database.username |DR_ID_DATABASE_USERNAME | Username for the H2 DR to ID database | sa
|dr.id.database.password |DR_ID_DATABASE_PASSWORD | Password for the H2 DR to ID database | sa
|dr.id.database.connection.pool.size |DR_ID_DATABASE_CONNECTION_POOL_SIZE | Size of the connection pool | 5
|dr.id.cache.size |DR_ID_CACHE_SIZE | Number of DR to ID entries to cache in memory for faster access | 1000
|file.monitor.database.name |FILE_MONITOR_DATABASE_NAME | Name of the file, without extension, of the H2 database in DATA_DIRECTORY to store file monitoring data | file_monitor
|file.monitor.database.username |FILE_MONITOR_DATABASE_USERNAME | Username for the H2 file monitoring database | sa
|file.monitor.database.password |FILE_MONITOR_DATABASE_PASSWORD | Password for the H2 file monitoring database | sa
|file.monitor.database.connection.pool.size |FILE_MONITOR_DATABASE_CONNECTION_POOL_SIZE | Size of the connection pool | 5
|n/a|AWS_REGION|When using AWS S3, the region to use. Required for S3.| None
|n/a|AWS_ACCESS_KEY_ID|Access Key ID for AWS S3. Required for S3.| None
|n/a|AWS_SECRET_ACCESS_KEY|Secret Access Key for AWS S3. Required for S3.| None


## Widgets Demo

See `example/widgets/readme.md`

## More information

See the Knowledge Base article: https://younite.us/resources/YOUnite-FS-Adaptor.html 